@@ -7,7 +7,7 @@ use warnings;
 
 use Carp;
 
-our $VERSION = '0.110_03';
+our $VERSION = '0.111_01';
 
 sub new {
     my ( $class ) = @_;
@@ -90,30 +90,28 @@ sub module_name {
 
 sub no_index {
     return +{
-      directory => [
-                     'doc',
-                     'examples',
-                     'inc',
-                     'tools',
-                     'xt',
-                   ],
-      file => [
-                'TODO.pod',
-              ],
+	directory => [ qw{ doc examples inc tools xt } ],
+	file	=> [ qw{ TODO.pod } ],
     };
 }
 
 sub provides {
-    -d 'lib'
-	or return;
+    my $provides;
     local $@ = undef;
-    my $provides = eval {
-	require Module::Metadata;
-	Module::Metadata->provides( version => 2, dir => 'lib' );
-    } or return;
 
     eval {
 	require CPAN::Meta;
+	require ExtUtils::Manifest;
+	require Module::Metadata;
+
+	my $manifest;
+	{
+	    local $SIG{__WARN__} = sub {};
+	    $manifest = ExtUtils::Manifest::maniread();
+	}
+	keys %{ $manifest || {} }
+	    or return;
+
 	# Skeleton so we can use should_index_file() and
 	# should_index_package().
 	my $meta = CPAN::Meta->new( {
@@ -122,11 +120,22 @@ sub provides {
 		no_index	=> no_index(),
 	    },
 	);
-	foreach my $pkg ( keys %{ $provides } ) {
-	    $meta->should_index_package( $pkg )
-		and $meta->should_index_file( $provides->{$pkg}{file} )
-		and next;
-	    delete $provides->{$pkg};
+
+	# The Module::Metadata docs say not to use
+	# package_versions_from_directory() directly, but the 'files =>'
+	# version of provides() is broken, and has been known to be so
+	# since 2014, so it's not getting fixed any time soon. So:
+
+	foreach my $fn ( sort keys %{ $manifest } ) {
+	    $fn =~ m/ [.] pm \z /smx
+		or next;
+	    my $pvd = Module::Metadata->package_versions_from_directory(
+		undef, [ $fn ] );
+	    foreach my $pkg ( keys %{ $pvd } ) {
+		$meta->should_index_package( $pkg )
+		    and $meta->should_index_file( $pvd->{$pkg}{file} )
+		    and $provides->{$pkg} = $pvd->{$pkg};
+	    }
 	}
 
 	1;
@@ -158,7 +167,7 @@ sub requires {
 
 sub recommended_module_versions {
     return (
-        'File::Which'   => 0,
+	# 'File::Which'   => 0,
     );
 }
 
